searchData={"items":[{"type":"module","title":"GeoSQL","doc":"GeoSQL provides access to GIS functions in SQL databases via Ecto. PostGIS 3.x and Spatialite are\ncurrently supported.\n\nSQL functions are sorted into modules by their appearance in standards (e.g. SQL/MM2 vs SQL/MM3),\ncategory or topic (such as topological and 3D functions), as well as their availability in\nspecific implementations. This includes the following modules:\n\n  * `GeoSQL.MM2`: functions from the SQL/MM v2 standard\n  * `GeoSQL.MM3`: functions from the SQL/MM v3 standard\n  * `GeoSQL.Common`: non-standard functions that are widely available\n  * `GeoSQL.PostGIS`: functions only found in PostGIS\n\nThis makes usage of feature sets self-documenting in the code, allowing developers to adopt or avoid\nfunctions that are not available in the databases the application uses.\n\nWhere there are subtle differences between implementations of the same functions, GeoSQL strives to\nhide those differences behind single function calls.\n\nThe `GeoSQL.Geometry` module provides access to geometric types in Ecto schemas.","ref":"GeoSQL.html"},{"type":"type","title":"GeoSQL.fragment/0","doc":"","ref":"GeoSQL.html#t:fragment/0"},{"type":"type","title":"GeoSQL.geometry_input/0","doc":"","ref":"GeoSQL.html#t:geometry_input/0"},{"type":"function","title":"GeoSQL.init/2","doc":"Performs runtime setup of an Ecto repo for use with GeoSQL. This can be done in the repo's\n`init/2` callback, or any time after it has been started.  This function is idempotent.\n\nIf using additional type extensions, pass those modules in via the `type_extensions` option.","ref":"GeoSQL.html#init/2"},{"type":"type","title":"GeoSQL.init_option/0","doc":"","ref":"GeoSQL.html#t:init_option/0"},{"type":"type","title":"GeoSQL.init_options/0","doc":"","ref":"GeoSQL.html#t:init_options/0"},{"type":"module","title":"GeoSQL.Geometry","doc":"Geometry types for use with ecto. Supported types include:\n\n  * GeoSQL.Geometry, a catch-all type\n  * GeoSQL.Geometry.Geopackage, a catch-all type for fields containing Geopackage geometry blobs\n  * GeoSQL.Geometry.Point\n  * GeoSQL.Geometry.PointZ\n  * GeoSQL.Geometry.PointM\n  * GeoSQL.Geometry.PointZM\n  * GeoSQL.Geometry.LineString\n  * GeoSQL.Geometry.LineStringZ\n  * GeoSQL.Geometry.LineStringZM\n  * GeoSQL.Geometry.Polygon\n  * GeoSQL.Geometry.PolygonZ\n  * GeoSQL.Geometry.MultiPoint\n  * GeoSQL.Geometry.MultiPointZ\n  * GeoSQL.Geometry.MultiLineString\n  * GeoSQL.Geometry.MultiLineStringZ\n  * GeoSQL.Geometry.MultiLineStringZM\n  * GeoSQL.Geometry.MultiPolygon\n  * GeoSQL.Geometry.MultiPolygonZ\n  * GeoSQL.Geometry.GeometryCollection\n\nExample:\n\n    defmodule MyApp.GeoTable do\n      use Ecto.Schema\n\n      schema \"specified_columns\" do\n        field(:name, :string)\n        field(:geometry, GeoSQL.Geometry) # will match any Geo type\n        field(:point, GeoSQL.Geometry.Point) # will reject any non-Point data\n        field(:linestring, GeoSQL.Geometry.LineStringZ) # will reject any non-LineStringZ data\n      end\n    end","ref":"GeoSQL.Geometry.html"},{"type":"function","title":"GeoSQL.Geometry.from_db_type/1","doc":"Returns the matching Geometry type for a geometry type returned from the database","ref":"GeoSQL.Geometry.html#from_db_type/1"},{"type":"type","title":"GeoSQL.Geometry.t/0","doc":"","ref":"GeoSQL.Geometry.html#t:t/0"},{"type":"module","title":"GeoSQL.Int4","doc":"A utility type to pin a number to a 32-bit integer, as required\nby some SQL functions. Can be used in Ecto schemas as `GeoSQL.Int4`\nor dynamically in queries such as `type(^layer.srid, GeoSQL.Int4)`.","ref":"GeoSQL.Int4.html"},{"type":"module","title":"GeoSQL.MM","doc":"Functions from the SQL/MM standard that can used in Ecto queries.","ref":"GeoSQL.MM.html"},{"type":"module","title":"Examples - GeoSQL.MM","doc":"defmodule Example do\n      import Ecto.Query\n      use GeoSQL.MM\n\n      def example_query(geom) do\n        from location in Location, limit: 5, select: MM.distance(location.geom, ^geom)\n      end\n    end","ref":"GeoSQL.MM.html#module-examples"},{"type":"macro","title":"GeoSQL.MM.area/1","doc":"","ref":"GeoSQL.MM.html#area/1"},{"type":"macro","title":"GeoSQL.MM.as_binary/1","doc":"","ref":"GeoSQL.MM.html#as_binary/1"},{"type":"macro","title":"GeoSQL.MM.as_text/1","doc":"","ref":"GeoSQL.MM.html#as_text/1"},{"type":"macro","title":"GeoSQL.MM.boundary/1","doc":"","ref":"GeoSQL.MM.html#boundary/1"},{"type":"type","title":"GeoSQL.MM.boundary_node_rule/0","doc":"","ref":"GeoSQL.MM.html#t:boundary_node_rule/0"},{"type":"macro","title":"GeoSQL.MM.buffer/2","doc":"","ref":"GeoSQL.MM.html#buffer/2"},{"type":"macro","title":"GeoSQL.MM.buffer/3","doc":"","ref":"GeoSQL.MM.html#buffer/3"},{"type":"macro","title":"GeoSQL.MM.centroid/1","doc":"","ref":"GeoSQL.MM.html#centroid/1"},{"type":"macro","title":"GeoSQL.MM.contains/2","doc":"","ref":"GeoSQL.MM.html#contains/2"},{"type":"macro","title":"GeoSQL.MM.convex_hull/1","doc":"","ref":"GeoSQL.MM.html#convex_hull/1"},{"type":"macro","title":"GeoSQL.MM.coord_dim/2","doc":"Note: this function takes an optional Ecto.Repo parameter due to\nsome backends implementing the non-standard ST_NDims rather than ST_CoordDim","ref":"GeoSQL.MM.html#coord_dim/2"},{"type":"macro","title":"GeoSQL.MM.crosses/2","doc":"","ref":"GeoSQL.MM.html#crosses/2"},{"type":"macro","title":"GeoSQL.MM.curve_n/2","doc":"","ref":"GeoSQL.MM.html#curve_n/2"},{"type":"macro","title":"GeoSQL.MM.curve_to_line/1","doc":"","ref":"GeoSQL.MM.html#curve_to_line/1"},{"type":"macro","title":"GeoSQL.MM.curve_to_line/4","doc":"","ref":"GeoSQL.MM.html#curve_to_line/4"},{"type":"macro","title":"GeoSQL.MM.difference/2","doc":"","ref":"GeoSQL.MM.html#difference/2"},{"type":"macro","title":"GeoSQL.MM.dimension/1","doc":"","ref":"GeoSQL.MM.html#dimension/1"},{"type":"macro","title":"GeoSQL.MM.disjoint/2","doc":"","ref":"GeoSQL.MM.html#disjoint/2"},{"type":"macro","title":"GeoSQL.MM.distance/3","doc":"","ref":"GeoSQL.MM.html#distance/3"},{"type":"macro","title":"GeoSQL.MM.end_point/1","doc":"","ref":"GeoSQL.MM.html#end_point/1"},{"type":"macro","title":"GeoSQL.MM.envelope/1","doc":"","ref":"GeoSQL.MM.html#envelope/1"},{"type":"macro","title":"GeoSQL.MM.equals/2","doc":"","ref":"GeoSQL.MM.html#equals/2"},{"type":"macro","title":"GeoSQL.MM.geom_collection_from_text/2","doc":"","ref":"GeoSQL.MM.html#geom_collection_from_text/2"},{"type":"macro","title":"GeoSQL.MM.geom_from_text/2","doc":"","ref":"GeoSQL.MM.html#geom_from_text/2"},{"type":"macro","title":"GeoSQL.MM.geom_from_wkb/2","doc":"","ref":"GeoSQL.MM.html#geom_from_wkb/2"},{"type":"macro","title":"GeoSQL.MM.geometry_n/2","doc":"","ref":"GeoSQL.MM.html#geometry_n/2"},{"type":"macro","title":"GeoSQL.MM.geometry_type/1","doc":"","ref":"GeoSQL.MM.html#geometry_type/1"},{"type":"macro","title":"GeoSQL.MM.gml_to_sql/1","doc":"","ref":"GeoSQL.MM.html#gml_to_sql/1"},{"type":"macro","title":"GeoSQL.MM.gml_to_sql/2","doc":"","ref":"GeoSQL.MM.html#gml_to_sql/2"},{"type":"macro","title":"GeoSQL.MM.interior_ring_n/2","doc":"","ref":"GeoSQL.MM.html#interior_ring_n/2"},{"type":"macro","title":"GeoSQL.MM.intersection/2","doc":"","ref":"GeoSQL.MM.html#intersection/2"},{"type":"macro","title":"GeoSQL.MM.intersects/2","doc":"","ref":"GeoSQL.MM.html#intersects/2"},{"type":"macro","title":"GeoSQL.MM.is_closed/1","doc":"","ref":"GeoSQL.MM.html#is_closed/1"},{"type":"macro","title":"GeoSQL.MM.is_empty/1","doc":"","ref":"GeoSQL.MM.html#is_empty/1"},{"type":"macro","title":"GeoSQL.MM.is_ring/1","doc":"","ref":"GeoSQL.MM.html#is_ring/1"},{"type":"macro","title":"GeoSQL.MM.is_simple/1","doc":"","ref":"GeoSQL.MM.html#is_simple/1"},{"type":"macro","title":"GeoSQL.MM.is_valid/1","doc":"","ref":"GeoSQL.MM.html#is_valid/1"},{"type":"macro","title":"GeoSQL.MM.length/1","doc":"","ref":"GeoSQL.MM.html#length/1"},{"type":"macro","title":"GeoSQL.MM.line_from_text/2","doc":"","ref":"GeoSQL.MM.html#line_from_text/2"},{"type":"macro","title":"GeoSQL.MM.linestring_from_wkb/2","doc":"","ref":"GeoSQL.MM.html#linestring_from_wkb/2"},{"type":"macro","title":"GeoSQL.MM.locate_along/2","doc":"","ref":"GeoSQL.MM.html#locate_along/2"},{"type":"macro","title":"GeoSQL.MM.locate_between/3","doc":"","ref":"GeoSQL.MM.html#locate_between/3"},{"type":"macro","title":"GeoSQL.MM.m/1","doc":"","ref":"GeoSQL.MM.html#m/1"},{"type":"macro","title":"GeoSQL.MM.m_line_from_text/2","doc":"","ref":"GeoSQL.MM.html#m_line_from_text/2"},{"type":"macro","title":"GeoSQL.MM.m_point_from_text/2","doc":"","ref":"GeoSQL.MM.html#m_point_from_text/2"},{"type":"macro","title":"GeoSQL.MM.m_poly_from_text/2","doc":"","ref":"GeoSQL.MM.html#m_poly_from_text/2"},{"type":"macro","title":"GeoSQL.MM.num_curves/1","doc":"","ref":"GeoSQL.MM.html#num_curves/1"},{"type":"macro","title":"GeoSQL.MM.num_geometries/1","doc":"","ref":"GeoSQL.MM.html#num_geometries/1"},{"type":"macro","title":"GeoSQL.MM.num_interior_rings/1","doc":"","ref":"GeoSQL.MM.html#num_interior_rings/1"},{"type":"macro","title":"GeoSQL.MM.num_patches/1","doc":"","ref":"GeoSQL.MM.html#num_patches/1"},{"type":"macro","title":"GeoSQL.MM.num_points/1","doc":"","ref":"GeoSQL.MM.html#num_points/1"},{"type":"macro","title":"GeoSQL.MM.ordering_equals/2","doc":"","ref":"GeoSQL.MM.html#ordering_equals/2"},{"type":"macro","title":"GeoSQL.MM.overlaps/2","doc":"","ref":"GeoSQL.MM.html#overlaps/2"},{"type":"macro","title":"GeoSQL.MM.patch_n/2","doc":"","ref":"GeoSQL.MM.html#patch_n/2"},{"type":"macro","title":"GeoSQL.MM.perimeter/1","doc":"","ref":"GeoSQL.MM.html#perimeter/1"},{"type":"macro","title":"GeoSQL.MM.perimeter/2","doc":"","ref":"GeoSQL.MM.html#perimeter/2"},{"type":"macro","title":"GeoSQL.MM.point/2","doc":"","ref":"GeoSQL.MM.html#point/2"},{"type":"macro","title":"GeoSQL.MM.point_from_text/2","doc":"","ref":"GeoSQL.MM.html#point_from_text/2"},{"type":"macro","title":"GeoSQL.MM.point_from_wkb/2","doc":"","ref":"GeoSQL.MM.html#point_from_wkb/2"},{"type":"macro","title":"GeoSQL.MM.point_n/2","doc":"","ref":"GeoSQL.MM.html#point_n/2"},{"type":"macro","title":"GeoSQL.MM.point_on_surface/1","doc":"","ref":"GeoSQL.MM.html#point_on_surface/1"},{"type":"macro","title":"GeoSQL.MM.polygon/2","doc":"NOTE: this function does not currently exist in Spatialite","ref":"GeoSQL.MM.html#polygon/2"},{"type":"macro","title":"GeoSQL.MM.polygon_from_text/2","doc":"","ref":"GeoSQL.MM.html#polygon_from_text/2"},{"type":"macro","title":"GeoSQL.MM.relate/2","doc":"","ref":"GeoSQL.MM.html#relate/2"},{"type":"macro","title":"GeoSQL.MM.relate/3","doc":"","ref":"GeoSQL.MM.html#relate/3"},{"type":"macro","title":"GeoSQL.MM.srid/1","doc":"","ref":"GeoSQL.MM.html#srid/1"},{"type":"macro","title":"GeoSQL.MM.start_point/1","doc":"","ref":"GeoSQL.MM.html#start_point/1"},{"type":"macro","title":"GeoSQL.MM.sym_difference/2","doc":"","ref":"GeoSQL.MM.html#sym_difference/2"},{"type":"macro","title":"GeoSQL.MM.touches/2","doc":"","ref":"GeoSQL.MM.html#touches/2"},{"type":"macro","title":"GeoSQL.MM.transform/2","doc":"","ref":"GeoSQL.MM.html#transform/2"},{"type":"macro","title":"GeoSQL.MM.union/1","doc":"","ref":"GeoSQL.MM.html#union/1"},{"type":"macro","title":"GeoSQL.MM.union/2","doc":"","ref":"GeoSQL.MM.html#union/2"},{"type":"macro","title":"GeoSQL.MM.within/2","doc":"","ref":"GeoSQL.MM.html#within/2"},{"type":"macro","title":"GeoSQL.MM.x/1","doc":"","ref":"GeoSQL.MM.html#x/1"},{"type":"macro","title":"GeoSQL.MM.y/1","doc":"","ref":"GeoSQL.MM.html#y/1"},{"type":"macro","title":"GeoSQL.MM.z/1","doc":"","ref":"GeoSQL.MM.html#z/1"},{"type":"module","title":"GeoSQL.MM.ThreeD","doc":"SQL/MM3 3D functions, typically prefixed with `ST_3D` or `CG_3D` in the standard.\n\nThis may require additional dependencies and configuration in the GIS store. For example,\nPostGIS requires the SFCGAL backend for some of these functions.","ref":"GeoSQL.MM.ThreeD.html"},{"type":"macro","title":"GeoSQL.MM.ThreeD.area/1","doc":"","ref":"GeoSQL.MM.ThreeD.html#area/1"},{"type":"macro","title":"GeoSQL.MM.ThreeD.d_within/3","doc":"","ref":"GeoSQL.MM.ThreeD.html#d_within/3"},{"type":"macro","title":"GeoSQL.MM.ThreeD.difference/2","doc":"","ref":"GeoSQL.MM.ThreeD.html#difference/2"},{"type":"macro","title":"GeoSQL.MM.ThreeD.distance/2","doc":"","ref":"GeoSQL.MM.ThreeD.html#distance/2"},{"type":"macro","title":"GeoSQL.MM.ThreeD.intersection/2","doc":"","ref":"GeoSQL.MM.ThreeD.html#intersection/2"},{"type":"macro","title":"GeoSQL.MM.ThreeD.intersects/2","doc":"","ref":"GeoSQL.MM.ThreeD.html#intersects/2"},{"type":"macro","title":"GeoSQL.MM.ThreeD.length/1","doc":"","ref":"GeoSQL.MM.ThreeD.html#length/1"},{"type":"macro","title":"GeoSQL.MM.ThreeD.perimeter/1","doc":"","ref":"GeoSQL.MM.ThreeD.html#perimeter/1"},{"type":"macro","title":"GeoSQL.MM.ThreeD.union/1","doc":"","ref":"GeoSQL.MM.ThreeD.html#union/1"},{"type":"macro","title":"GeoSQL.MM.ThreeD.union/2","doc":"","ref":"GeoSQL.MM.ThreeD.html#union/2"},{"type":"macro","title":"GeoSQL.MM.ThreeD.volume/1","doc":"","ref":"GeoSQL.MM.ThreeD.html#volume/1"},{"type":"module","title":"GeoSQL.MM.Topo","doc":"SQL/MM3 Topological functions which can be applied to a toplogy object.\n\nTopology functions which operate on geometries that are passed into them rather than a\ntopology object are found in the `GeoSQL.MM3` module.\n\nThese function may also require additional dependencies and configuration in the GIS store.\nFor example, PostGIS requires the SFCGAL backend for some of these functions.","ref":"GeoSQL.MM.Topo.html"},{"type":"macro","title":"GeoSQL.MM.Topo.add_edge_mod_face/5","doc":"","ref":"GeoSQL.MM.Topo.html#add_edge_mod_face/5"},{"type":"macro","title":"GeoSQL.MM.Topo.add_edge_new_faces/5","doc":"","ref":"GeoSQL.MM.Topo.html#add_edge_new_faces/5"},{"type":"macro","title":"GeoSQL.MM.Topo.add_iso_edge/5","doc":"","ref":"GeoSQL.MM.Topo.html#add_iso_edge/5"},{"type":"macro","title":"GeoSQL.MM.Topo.add_iso_node/4","doc":"","ref":"GeoSQL.MM.Topo.html#add_iso_node/4"},{"type":"macro","title":"GeoSQL.MM.Topo.change_edge_gome/4","doc":"","ref":"GeoSQL.MM.Topo.html#change_edge_gome/4"},{"type":"macro","title":"GeoSQL.MM.Topo.create_topo_geo/2","doc":"","ref":"GeoSQL.MM.Topo.html#create_topo_geo/2"},{"type":"macro","title":"GeoSQL.MM.Topo.get_face_edges/3","doc":"","ref":"GeoSQL.MM.Topo.html#get_face_edges/3"},{"type":"macro","title":"GeoSQL.MM.Topo.get_face_geometry/3","doc":"","ref":"GeoSQL.MM.Topo.html#get_face_geometry/3"},{"type":"macro","title":"GeoSQL.MM.Topo.init_topo_geo/2","doc":"","ref":"GeoSQL.MM.Topo.html#init_topo_geo/2"},{"type":"macro","title":"GeoSQL.MM.Topo.mod_edge_heal/4","doc":"","ref":"GeoSQL.MM.Topo.html#mod_edge_heal/4"},{"type":"macro","title":"GeoSQL.MM.Topo.mod_edge_split/4","doc":"","ref":"GeoSQL.MM.Topo.html#mod_edge_split/4"},{"type":"macro","title":"GeoSQL.MM.Topo.move_iso_node/4","doc":"","ref":"GeoSQL.MM.Topo.html#move_iso_node/4"},{"type":"macro","title":"GeoSQL.MM.Topo.new_edge_heal/4","doc":"","ref":"GeoSQL.MM.Topo.html#new_edge_heal/4"},{"type":"macro","title":"GeoSQL.MM.Topo.new_edge_split/4","doc":"","ref":"GeoSQL.MM.Topo.html#new_edge_split/4"},{"type":"macro","title":"GeoSQL.MM.Topo.rem_edge_mod_face/3","doc":"","ref":"GeoSQL.MM.Topo.html#rem_edge_mod_face/3"},{"type":"macro","title":"GeoSQL.MM.Topo.rem_edge_new_face/3","doc":"","ref":"GeoSQL.MM.Topo.html#rem_edge_new_face/3"},{"type":"macro","title":"GeoSQL.MM.Topo.remove_iso_edge/3","doc":"","ref":"GeoSQL.MM.Topo.html#remove_iso_edge/3"},{"type":"macro","title":"GeoSQL.MM.Topo.remove_iso_node/3","doc":"","ref":"GeoSQL.MM.Topo.html#remove_iso_node/3"},{"type":"module","title":"GeoSQL.Common","doc":"Commonly supported, but non-standard, GIS SQL functions. These are\nfound in multiple database implementations, though they may differ in minor\nsyntactical details in each implementation.","ref":"GeoSQL.Common.html"},{"type":"module","title":"Implementation differences - GeoSQL.Common","doc":"In cases where the generated SQL differs based on the database in use, those\nfunctions will take an optional `Ecto.Repo` argument. When provided, this allows\nthe function to decide which implementation-specific ipmlementation to use.\n\nWhen no `Ecto.Repo` is provided, functions default to `PostGIS`-compatible syntax.","ref":"GeoSQL.Common.html#module-implementation-differences"},{"type":"module","title":"Database support - GeoSQL.Common","doc":"The features in the `GeoSQL.Common` modules are intended to contain functions\navailable in all support SQL GIS extensions.\n\nSome backends may require special initialization or\ndependencies loaded for some of these functions to work. For example, SpatiaLite\nmust be built with the GEO package for some of these functions to be available.\n\nConsult the documentation for the database GIS extension being used for such\nrequirements.","ref":"GeoSQL.Common.html#module-database-support"},{"type":"macro","title":"GeoSQL.Common.add_measure/3","doc":"","ref":"GeoSQL.Common.html#add_measure/3"},{"type":"macro","title":"GeoSQL.Common.add_point/3","doc":"","ref":"GeoSQL.Common.html#add_point/3"},{"type":"macro","title":"GeoSQL.Common.as_ewkb/2","doc":"","ref":"GeoSQL.Common.html#as_ewkb/2"},{"type":"macro","title":"GeoSQL.Common.as_ewkt/2","doc":"","ref":"GeoSQL.Common.html#as_ewkt/2"},{"type":"macro","title":"GeoSQL.Common.as_geojson/2","doc":"","ref":"GeoSQL.Common.html#as_geojson/2"},{"type":"macro","title":"GeoSQL.Common.as_gml/4","doc":"","ref":"GeoSQL.Common.html#as_gml/4"},{"type":"macro","title":"GeoSQL.Common.as_kml/4","doc":"","ref":"GeoSQL.Common.html#as_kml/4"},{"type":"macro","title":"GeoSQL.Common.azimuth/2","doc":"","ref":"GeoSQL.Common.html#azimuth/2"},{"type":"macro","title":"GeoSQL.Common.bd_m_poly_from_text/2","doc":"","ref":"GeoSQL.Common.html#bd_m_poly_from_text/2"},{"type":"macro","title":"GeoSQL.Common.bd_poly_from_text/2","doc":"","ref":"GeoSQL.Common.html#bd_poly_from_text/2"},{"type":"macro","title":"GeoSQL.Common.build_area/1","doc":"","ref":"GeoSQL.Common.html#build_area/1"},{"type":"macro","title":"GeoSQL.Common.closest_point/4","doc":"","ref":"GeoSQL.Common.html#closest_point/4"},{"type":"macro","title":"GeoSQL.Common.collect/1","doc":"","ref":"GeoSQL.Common.html#collect/1"},{"type":"macro","title":"GeoSQL.Common.collect/2","doc":"","ref":"GeoSQL.Common.html#collect/2"},{"type":"macro","title":"GeoSQL.Common.collection_extract/2","doc":"","ref":"GeoSQL.Common.html#collection_extract/2"},{"type":"macro","title":"GeoSQL.Common.concave_hull/3","doc":"","ref":"GeoSQL.Common.html#concave_hull/3"},{"type":"macro","title":"GeoSQL.Common.covered_by/2","doc":"","ref":"GeoSQL.Common.html#covered_by/2"},{"type":"macro","title":"GeoSQL.Common.covers/2","doc":"","ref":"GeoSQL.Common.html#covers/2"},{"type":"macro","title":"GeoSQL.Common.estimated_extent/3","doc":"","ref":"GeoSQL.Common.html#estimated_extent/3"},{"type":"macro","title":"GeoSQL.Common.expand/2","doc":"","ref":"GeoSQL.Common.html#expand/2"},{"type":"macro","title":"GeoSQL.Common.extent/2","doc":"","ref":"GeoSQL.Common.html#extent/2"},{"type":"macro","title":"GeoSQL.Common.exterior_ring/1","doc":"","ref":"GeoSQL.Common.html#exterior_ring/1"},{"type":"macro","title":"GeoSQL.Common.flip_coordinates/2","doc":"","ref":"GeoSQL.Common.html#flip_coordinates/2"},{"type":"macro","title":"GeoSQL.Common.geom_from_ewkb/2","doc":"","ref":"GeoSQL.Common.html#geom_from_ewkb/2"},{"type":"macro","title":"GeoSQL.Common.geom_from_ewkt/2","doc":"","ref":"GeoSQL.Common.html#geom_from_ewkt/2"},{"type":"macro","title":"GeoSQL.Common.geom_from_geojson/2","doc":"","ref":"GeoSQL.Common.html#geom_from_geojson/2"},{"type":"macro","title":"GeoSQL.Common.geom_from_gml/3","doc":"Passing an srid of 0 (the default) will not set an srid on the results. It is ignored for Spatialite in all cases.","ref":"GeoSQL.Common.html#geom_from_gml/3"},{"type":"macro","title":"GeoSQL.Common.geom_from_kml/2","doc":"","ref":"GeoSQL.Common.html#geom_from_kml/2"},{"type":"macro","title":"GeoSQL.Common.interpolate_point/2","doc":"","ref":"GeoSQL.Common.html#interpolate_point/2"},{"type":"macro","title":"GeoSQL.Common.is_polygon_clockwise/1","doc":"","ref":"GeoSQL.Common.html#is_polygon_clockwise/1"},{"type":"macro","title":"GeoSQL.Common.is_polygon_counter_clockwise/1","doc":"","ref":"GeoSQL.Common.html#is_polygon_counter_clockwise/1"},{"type":"macro","title":"GeoSQL.Common.is_valid_detail/2","doc":"","ref":"GeoSQL.Common.html#is_valid_detail/2"},{"type":"macro","title":"GeoSQL.Common.is_valid_reason/2","doc":"","ref":"GeoSQL.Common.html#is_valid_reason/2"},{"type":"macro","title":"GeoSQL.Common.largest_empty_circle/3","doc":"","ref":"GeoSQL.Common.html#largest_empty_circle/3"},{"type":"macro","title":"GeoSQL.Common.line_interpolate_point/4","doc":"Spatialite does not support spheroid calculations, and therefore `use_spheroid?` is ignored\non that platform.","ref":"GeoSQL.Common.html#line_interpolate_point/4"},{"type":"macro","title":"GeoSQL.Common.line_interpolate_points/4","doc":"Note that Spatialite does not support spheroid calculations.","ref":"GeoSQL.Common.html#line_interpolate_points/4"},{"type":"macro","title":"GeoSQL.Common.line_locate_point/4","doc":"Spatialite does not support spheroid calculations, and therefore `use_spheroid?` is ignored\non that platform.","ref":"GeoSQL.Common.html#line_locate_point/4"},{"type":"macro","title":"GeoSQL.Common.line_merge/3","doc":"","ref":"GeoSQL.Common.html#line_merge/3"},{"type":"macro","title":"GeoSQL.Common.line_substring/4","doc":"","ref":"GeoSQL.Common.html#line_substring/4"},{"type":"macro","title":"GeoSQL.Common.locate_along/2","doc":"","ref":"GeoSQL.Common.html#locate_along/2"},{"type":"macro","title":"GeoSQL.Common.locate_between/3","doc":"","ref":"GeoSQL.Common.html#locate_between/3"},{"type":"macro","title":"GeoSQL.Common.make_point/3","doc":"","ref":"GeoSQL.Common.html#make_point/3"},{"type":"macro","title":"GeoSQL.Common.make_point_m/4","doc":"","ref":"GeoSQL.Common.html#make_point_m/4"},{"type":"macro","title":"GeoSQL.Common.make_point_z/4","doc":"","ref":"GeoSQL.Common.html#make_point_z/4"},{"type":"macro","title":"GeoSQL.Common.make_point_zm/5","doc":"","ref":"GeoSQL.Common.html#make_point_zm/5"},{"type":"macro","title":"GeoSQL.Common.make_valid/1","doc":"","ref":"GeoSQL.Common.html#make_valid/1"},{"type":"macro","title":"GeoSQL.Common.max_coord/3","doc":"","ref":"GeoSQL.Common.html#max_coord/3"},{"type":"macro","title":"GeoSQL.Common.max_distance/2","doc":"","ref":"GeoSQL.Common.html#max_distance/2"},{"type":"macro","title":"GeoSQL.Common.maximum_inscribed_circle/2","doc":"","ref":"GeoSQL.Common.html#maximum_inscribed_circle/2"},{"type":"macro","title":"GeoSQL.Common.min_coord/3","doc":"","ref":"GeoSQL.Common.html#min_coord/3"},{"type":"macro","title":"GeoSQL.Common.minimum_bounding_circle/2","doc":"","ref":"GeoSQL.Common.html#minimum_bounding_circle/2"},{"type":"macro","title":"GeoSQL.Common.minimum_bounding_radius/2","doc":"","ref":"GeoSQL.Common.html#minimum_bounding_radius/2"},{"type":"macro","title":"GeoSQL.Common.minimum_clearance/2","doc":"","ref":"GeoSQL.Common.html#minimum_clearance/2"},{"type":"macro","title":"GeoSQL.Common.minimum_clearance_line/2","doc":"","ref":"GeoSQL.Common.html#minimum_clearance_line/2"},{"type":"macro","title":"GeoSQL.Common.multi/1","doc":"","ref":"GeoSQL.Common.html#multi/1"},{"type":"macro","title":"GeoSQL.Common.node/1","doc":"","ref":"GeoSQL.Common.html#node/1"},{"type":"macro","title":"GeoSQL.Common.number_of_dimensions/1","doc":"","ref":"GeoSQL.Common.html#number_of_dimensions/1"},{"type":"macro","title":"GeoSQL.Common.number_of_geometries/1","doc":"","ref":"GeoSQL.Common.html#number_of_geometries/1"},{"type":"macro","title":"GeoSQL.Common.number_of_points/1","doc":"","ref":"GeoSQL.Common.html#number_of_points/1"},{"type":"macro","title":"GeoSQL.Common.number_of_rings/1","doc":"","ref":"GeoSQL.Common.html#number_of_rings/1"},{"type":"macro","title":"GeoSQL.Common.offset_curve/2","doc":"","ref":"GeoSQL.Common.html#offset_curve/2"},{"type":"macro","title":"GeoSQL.Common.oriented_envelope/1","doc":"","ref":"GeoSQL.Common.html#oriented_envelope/1"},{"type":"macro","title":"GeoSQL.Common.polygonize/1","doc":"","ref":"GeoSQL.Common.html#polygonize/1"},{"type":"macro","title":"GeoSQL.Common.project/3","doc":"","ref":"GeoSQL.Common.html#project/3"},{"type":"macro","title":"GeoSQL.Common.reduce_precision/2","doc":"","ref":"GeoSQL.Common.html#reduce_precision/2"},{"type":"macro","title":"GeoSQL.Common.relate_match/2","doc":"","ref":"GeoSQL.Common.html#relate_match/2"},{"type":"macro","title":"GeoSQL.Common.remove_point/2","doc":"","ref":"GeoSQL.Common.html#remove_point/2"},{"type":"macro","title":"GeoSQL.Common.remove_repeated_points/3","doc":"","ref":"GeoSQL.Common.html#remove_repeated_points/3"},{"type":"macro","title":"GeoSQL.Common.reverse/1","doc":"","ref":"GeoSQL.Common.html#reverse/1"},{"type":"macro","title":"GeoSQL.Common.rotate/3","doc":"","ref":"GeoSQL.Common.html#rotate/3"},{"type":"macro","title":"GeoSQL.Common.scale/4","doc":"","ref":"GeoSQL.Common.html#scale/4"},{"type":"macro","title":"GeoSQL.Common.segmentize/2","doc":"","ref":"GeoSQL.Common.html#segmentize/2"},{"type":"macro","title":"GeoSQL.Common.set_point/3","doc":"","ref":"GeoSQL.Common.html#set_point/3"},{"type":"macro","title":"GeoSQL.Common.set_srid/3","doc":"","ref":"GeoSQL.Common.html#set_srid/3"},{"type":"macro","title":"GeoSQL.Common.shared_paths/2","doc":"","ref":"GeoSQL.Common.html#shared_paths/2"},{"type":"macro","title":"GeoSQL.Common.shift_longitude/2","doc":"","ref":"GeoSQL.Common.html#shift_longitude/2"},{"type":"macro","title":"GeoSQL.Common.shortest_line/4","doc":"","ref":"GeoSQL.Common.html#shortest_line/4"},{"type":"macro","title":"GeoSQL.Common.simplify/2","doc":"","ref":"GeoSQL.Common.html#simplify/2"},{"type":"macro","title":"GeoSQL.Common.simplify_preserve_topology/2","doc":"","ref":"GeoSQL.Common.html#simplify_preserve_topology/2"},{"type":"macro","title":"GeoSQL.Common.snap/3","doc":"","ref":"GeoSQL.Common.html#snap/3"},{"type":"macro","title":"GeoSQL.Common.snap_to_grid/2","doc":"","ref":"GeoSQL.Common.html#snap_to_grid/2"},{"type":"macro","title":"GeoSQL.Common.snap_to_grid/3","doc":"","ref":"GeoSQL.Common.html#snap_to_grid/3"},{"type":"macro","title":"GeoSQL.Common.snap_to_grid/5","doc":"","ref":"GeoSQL.Common.html#snap_to_grid/5"},{"type":"macro","title":"GeoSQL.Common.snap_to_grid/6","doc":"","ref":"GeoSQL.Common.html#snap_to_grid/6"},{"type":"macro","title":"GeoSQL.Common.split/2","doc":"","ref":"GeoSQL.Common.html#split/2"},{"type":"macro","title":"GeoSQL.Common.subdivide/2","doc":"","ref":"GeoSQL.Common.html#subdivide/2"},{"type":"macro","title":"GeoSQL.Common.transform_pipeline/4","doc":"","ref":"GeoSQL.Common.html#transform_pipeline/4"},{"type":"macro","title":"GeoSQL.Common.translate/4","doc":"","ref":"GeoSQL.Common.html#translate/4"},{"type":"macro","title":"GeoSQL.Common.triangulate_polygon/2","doc":"","ref":"GeoSQL.Common.html#triangulate_polygon/2"},{"type":"macro","title":"GeoSQL.Common.unary_union/1","doc":"","ref":"GeoSQL.Common.html#unary_union/1"},{"type":"module","title":"GeoSQL.Common.ThreeD","doc":"Non-standard but commonly implemented 3D functions","ref":"GeoSQL.Common.ThreeD.html"},{"type":"macro","title":"GeoSQL.Common.ThreeD.max_distance/2","doc":"","ref":"GeoSQL.Common.ThreeD.html#max_distance/2"},{"type":"module","title":"GeoSQL.Common.Topo","doc":"Non-standard but commonly implement Topological functions which can be applied to a toplogy object.\n\nTopology functions which operate on geometries that are passed into them rather than a\ntopology object are found in the `GeoSQL.Common` module.","ref":"GeoSQL.Common.Topo.html"},{"type":"macro","title":"GeoSQL.Common.Topo.add_line_string/4","doc":"","ref":"GeoSQL.Common.Topo.html#add_line_string/4"},{"type":"macro","title":"GeoSQL.Common.Topo.add_node/4","doc":"","ref":"GeoSQL.Common.Topo.html#add_node/4"},{"type":"macro","title":"GeoSQL.Common.Topo.get_edge_by_point/4","doc":"","ref":"GeoSQL.Common.Topo.html#get_edge_by_point/4"},{"type":"macro","title":"GeoSQL.Common.Topo.get_face_by_point/4","doc":"","ref":"GeoSQL.Common.Topo.html#get_face_by_point/4"},{"type":"macro","title":"GeoSQL.Common.Topo.get_face_edges/3","doc":"","ref":"GeoSQL.Common.Topo.html#get_face_edges/3"},{"type":"macro","title":"GeoSQL.Common.Topo.get_face_geometry/3","doc":"","ref":"GeoSQL.Common.Topo.html#get_face_geometry/3"},{"type":"macro","title":"GeoSQL.Common.Topo.get_node_by_point/4","doc":"","ref":"GeoSQL.Common.Topo.html#get_node_by_point/4"},{"type":"macro","title":"GeoSQL.Common.Topo.polygonize/3","doc":"","ref":"GeoSQL.Common.Topo.html#polygonize/3"},{"type":"module","title":"GeoSQL.PostGIS","doc":"Non-standard GIS functions found in PostGIS.","ref":"GeoSQL.PostGIS.html"},{"type":"macro","title":"GeoSQL.PostGIS.affine/7","doc":"","ref":"GeoSQL.PostGIS.html#affine/7"},{"type":"macro","title":"GeoSQL.PostGIS.affine/13","doc":"","ref":"GeoSQL.PostGIS.html#affine/13"},{"type":"macro","title":"GeoSQL.PostGIS.angle/2","doc":"","ref":"GeoSQL.PostGIS.html#angle/2"},{"type":"macro","title":"GeoSQL.PostGIS.bounding_diagonal/2","doc":"","ref":"GeoSQL.PostGIS.html#bounding_diagonal/2"},{"type":"macro","title":"GeoSQL.PostGIS.collection_homogenize/1","doc":"","ref":"GeoSQL.PostGIS.html#collection_homogenize/1"},{"type":"macro","title":"GeoSQL.PostGIS.contains_properly/3","doc":"","ref":"GeoSQL.PostGIS.html#contains_properly/3"},{"type":"macro","title":"GeoSQL.PostGIS.d_fully_within/3","doc":"","ref":"GeoSQL.PostGIS.html#d_fully_within/3"},{"type":"macro","title":"GeoSQL.PostGIS.d_within/4","doc":"","ref":"GeoSQL.PostGIS.html#d_within/4"},{"type":"macro","title":"GeoSQL.PostGIS.distance_sphere/2","doc":"","ref":"GeoSQL.PostGIS.html#distance_sphere/2"},{"type":"macro","title":"GeoSQL.PostGIS.dump/1","doc":"","ref":"GeoSQL.PostGIS.html#dump/1"},{"type":"macro","title":"GeoSQL.PostGIS.dump_points/1","doc":"","ref":"GeoSQL.PostGIS.html#dump_points/1"},{"type":"macro","title":"GeoSQL.PostGIS.dump_rings/1","doc":"","ref":"GeoSQL.PostGIS.html#dump_rings/1"},{"type":"macro","title":"GeoSQL.PostGIS.dump_segments/1","doc":"","ref":"GeoSQL.PostGIS.html#dump_segments/1"},{"type":"macro","title":"GeoSQL.PostGIS.expand/3","doc":"","ref":"GeoSQL.PostGIS.html#expand/3"},{"type":"macro","title":"GeoSQL.PostGIS.expand/4","doc":"","ref":"GeoSQL.PostGIS.html#expand/4"},{"type":"macro","title":"GeoSQL.PostGIS.expand/5","doc":"","ref":"GeoSQL.PostGIS.html#expand/5"},{"type":"macro","title":"GeoSQL.PostGIS.generate_points/2","doc":"","ref":"GeoSQL.PostGIS.html#generate_points/2"},{"type":"macro","title":"GeoSQL.PostGIS.generate_points/3","doc":"","ref":"GeoSQL.PostGIS.html#generate_points/3"},{"type":"macro","title":"GeoSQL.PostGIS.has_arc/1","doc":"","ref":"GeoSQL.PostGIS.html#has_arc/1"},{"type":"macro","title":"GeoSQL.PostGIS.has_m/1","doc":"","ref":"GeoSQL.PostGIS.html#has_m/1"},{"type":"macro","title":"GeoSQL.PostGIS.has_z/1","doc":"","ref":"GeoSQL.PostGIS.html#has_z/1"},{"type":"macro","title":"GeoSQL.PostGIS.inverse_transform_pipeline/3","doc":"","ref":"GeoSQL.PostGIS.html#inverse_transform_pipeline/3"},{"type":"macro","title":"GeoSQL.PostGIS.is_collection/1","doc":"","ref":"GeoSQL.PostGIS.html#is_collection/1"},{"type":"macro","title":"GeoSQL.PostGIS.line_crossing_direction/2","doc":"","ref":"GeoSQL.PostGIS.html#line_crossing_direction/2"},{"type":"macro","title":"GeoSQL.PostGIS.line_to_curve/1","doc":"","ref":"GeoSQL.PostGIS.html#line_to_curve/1"},{"type":"macro","title":"GeoSQL.PostGIS.locate_along/3","doc":"PostGIS extension to `locate_along` with an offset","ref":"GeoSQL.PostGIS.html#locate_along/3"},{"type":"macro","title":"GeoSQL.PostGIS.locate_between/4","doc":"PostGIS extension to `locate_between` with an offset","ref":"GeoSQL.PostGIS.html#locate_between/4"},{"type":"macro","title":"GeoSQL.PostGIS.locate_between_elevations/3","doc":"","ref":"GeoSQL.PostGIS.html#locate_between_elevations/3"},{"type":"macro","title":"GeoSQL.PostGIS.longest_line/2","doc":"","ref":"GeoSQL.PostGIS.html#longest_line/2"},{"type":"macro","title":"GeoSQL.PostGIS.make_box_2d/2","doc":"","ref":"GeoSQL.PostGIS.html#make_box_2d/2"},{"type":"macro","title":"GeoSQL.PostGIS.make_envelope/4","doc":"","ref":"GeoSQL.PostGIS.html#make_envelope/4"},{"type":"macro","title":"GeoSQL.PostGIS.make_envelope/5","doc":"","ref":"GeoSQL.PostGIS.html#make_envelope/5"},{"type":"macro","title":"GeoSQL.PostGIS.make_valid/2","doc":"","ref":"GeoSQL.PostGIS.html#make_valid/2"},{"type":"macro","title":"GeoSQL.PostGIS.mem_size/1","doc":"","ref":"GeoSQL.PostGIS.html#mem_size/1"},{"type":"macro","title":"GeoSQL.PostGIS.mem_union/1","doc":"","ref":"GeoSQL.PostGIS.html#mem_union/1"},{"type":"macro","title":"GeoSQL.PostGIS.normalize/1","doc":"","ref":"GeoSQL.PostGIS.html#normalize/1"},{"type":"macro","title":"GeoSQL.PostGIS.points/1","doc":"","ref":"GeoSQL.PostGIS.html#points/1"},{"type":"type","title":"GeoSQL.PostGIS.quantize_coordinate/0","doc":"","ref":"GeoSQL.PostGIS.html#t:quantize_coordinate/0"},{"type":"macro","title":"GeoSQL.PostGIS.quantize_coordinates/2","doc":"","ref":"GeoSQL.PostGIS.html#quantize_coordinates/2"},{"type":"type","title":"GeoSQL.PostGIS.quantize_precision/0","doc":"","ref":"GeoSQL.PostGIS.html#t:quantize_precision/0"},{"type":"macro","title":"GeoSQL.PostGIS.remove_irrelevant_points_for_view/3","doc":"","ref":"GeoSQL.PostGIS.html#remove_irrelevant_points_for_view/3"},{"type":"macro","title":"GeoSQL.PostGIS.remove_small_parts/3","doc":"","ref":"GeoSQL.PostGIS.html#remove_small_parts/3"},{"type":"macro","title":"GeoSQL.PostGIS.rotate/2","doc":"","ref":"GeoSQL.PostGIS.html#rotate/2"},{"type":"macro","title":"GeoSQL.PostGIS.rotate/3","doc":"","ref":"GeoSQL.PostGIS.html#rotate/3"},{"type":"macro","title":"GeoSQL.PostGIS.rotate_x/2","doc":"","ref":"GeoSQL.PostGIS.html#rotate_x/2"},{"type":"macro","title":"GeoSQL.PostGIS.rotate_y/2","doc":"","ref":"GeoSQL.PostGIS.html#rotate_y/2"},{"type":"macro","title":"GeoSQL.PostGIS.rotate_z/2","doc":"","ref":"GeoSQL.PostGIS.html#rotate_z/2"},{"type":"macro","title":"GeoSQL.PostGIS.scale/2","doc":"","ref":"GeoSQL.PostGIS.html#scale/2"},{"type":"macro","title":"GeoSQL.PostGIS.scale/3","doc":"","ref":"GeoSQL.PostGIS.html#scale/3"},{"type":"macro","title":"GeoSQL.PostGIS.scale/4","doc":"","ref":"GeoSQL.PostGIS.html#scale/4"},{"type":"macro","title":"GeoSQL.PostGIS.scroll/2","doc":"","ref":"GeoSQL.PostGIS.html#scroll/2"},{"type":"macro","title":"GeoSQL.PostGIS.summary/1","doc":"","ref":"GeoSQL.PostGIS.html#summary/1"},{"type":"macro","title":"GeoSQL.PostGIS.swap_ordinates/2","doc":"","ref":"GeoSQL.PostGIS.html#swap_ordinates/2"},{"type":"macro","title":"GeoSQL.PostGIS.trans_scale/5","doc":"","ref":"GeoSQL.PostGIS.html#trans_scale/5"},{"type":"macro","title":"GeoSQL.PostGIS.wrap_x/3","doc":"","ref":"GeoSQL.PostGIS.html#wrap_x/3"},{"type":"macro","title":"GeoSQL.PostGIS.zm_flag/1","doc":"","ref":"GeoSQL.PostGIS.html#zm_flag/1"},{"type":"module","title":"GeoSQL.PostGIS.Extension","doc":"A type extension for PostGIS data in PostgreSQL databases.\n\nAutomatically applied when a PostgreSQL Ecto repo is passed into `GeoSQL.init/2`.","ref":"GeoSQL.PostGIS.Extension.html"},{"type":"function","title":"GeoSQL.PostGIS.Extension.extensions/0","doc":"Convenience function to get the extension modules for PostGIS support.","ref":"GeoSQL.PostGIS.Extension.html#extensions/0"},{"type":"module","title":"GeoSQL.PostGIS.Operators","doc":"Non-standard GIS operators found in PostGIS.\n\n  These are often more optimized and/or specialized than their ST_* equivalents.","ref":"GeoSQL.PostGIS.Operators.html"},{"type":"macro","title":"GeoSQL.PostGIS.Operators.bbox_equals?/2","doc":"","ref":"GeoSQL.PostGIS.Operators.html#bbox_equals?/2"},{"type":"macro","title":"GeoSQL.PostGIS.Operators.bbox_intersects?/2","doc":"","ref":"GeoSQL.PostGIS.Operators.html#bbox_intersects?/2"},{"type":"macro","title":"GeoSQL.PostGIS.Operators.bbox_nd_intersects?/2","doc":"","ref":"GeoSQL.PostGIS.Operators.html#bbox_nd_intersects?/2"},{"type":"macro","title":"GeoSQL.PostGIS.Operators.contained_by?/2","doc":"","ref":"GeoSQL.PostGIS.Operators.html#contained_by?/2"},{"type":"macro","title":"GeoSQL.PostGIS.Operators.contains?/2","doc":"","ref":"GeoSQL.PostGIS.Operators.html#contains?/2"},{"type":"macro","title":"GeoSQL.PostGIS.Operators.distance_2d/2","doc":"","ref":"GeoSQL.PostGIS.Operators.html#distance_2d/2"},{"type":"macro","title":"GeoSQL.PostGIS.Operators.distance_at_closest_approach/2","doc":"","ref":"GeoSQL.PostGIS.Operators.html#distance_at_closest_approach/2"},{"type":"macro","title":"GeoSQL.PostGIS.Operators.distance_between_bboxes/2","doc":"","ref":"GeoSQL.PostGIS.Operators.html#distance_between_bboxes/2"},{"type":"macro","title":"GeoSQL.PostGIS.Operators.distance_nd/2","doc":"","ref":"GeoSQL.PostGIS.Operators.html#distance_nd/2"},{"type":"macro","title":"GeoSQL.PostGIS.Operators.equal?/2","doc":"","ref":"GeoSQL.PostGIS.Operators.html#equal?/2"},{"type":"macro","title":"GeoSQL.PostGIS.Operators.overlaps_or_to_the?/3","doc":"","ref":"GeoSQL.PostGIS.Operators.html#overlaps_or_to_the?/3"},{"type":"macro","title":"GeoSQL.PostGIS.Operators.strictly_to_the?/3","doc":"","ref":"GeoSQL.PostGIS.Operators.html#strictly_to_the?/3"},{"type":"module","title":"GeoSQL.PostGIS.ThreeD","doc":"Non-standard PostGIS 3D functions","ref":"GeoSQL.PostGIS.ThreeD.html"},{"type":"macro","title":"GeoSQL.PostGIS.ThreeD.closest_point/2","doc":"","ref":"GeoSQL.PostGIS.ThreeD.html#closest_point/2"},{"type":"macro","title":"GeoSQL.PostGIS.ThreeD.d_fully_within/3","doc":"","ref":"GeoSQL.PostGIS.ThreeD.html#d_fully_within/3"},{"type":"macro","title":"GeoSQL.PostGIS.ThreeD.extent/1","doc":"","ref":"GeoSQL.PostGIS.ThreeD.html#extent/1"},{"type":"macro","title":"GeoSQL.PostGIS.ThreeD.line_interpolate_point/2","doc":"","ref":"GeoSQL.PostGIS.ThreeD.html#line_interpolate_point/2"},{"type":"macro","title":"GeoSQL.PostGIS.ThreeD.longest_line/2","doc":"","ref":"GeoSQL.PostGIS.ThreeD.html#longest_line/2"},{"type":"macro","title":"GeoSQL.PostGIS.ThreeD.make_box/2","doc":"","ref":"GeoSQL.PostGIS.ThreeD.html#make_box/2"},{"type":"macro","title":"GeoSQL.PostGIS.ThreeD.shortest_line/2","doc":"","ref":"GeoSQL.PostGIS.ThreeD.html#shortest_line/2"},{"type":"module","title":"GeoSQL.PostGIS.Topo","doc":"Topological functions implemented in PostGIS which can be applied to a toplogy object.\n\nTopology functions which operate on geometries that are passed into them rather than a\ntopology object are found in the `GeoSQL.PostGIS` module.","ref":"GeoSQL.PostGIS.Topo.html"},{"type":"macro","title":"GeoSQL.PostGIS.Topo.add_face/3","doc":"","ref":"GeoSQL.PostGIS.Topo.html#add_face/3"},{"type":"macro","title":"GeoSQL.PostGIS.Topo.add_node/4","doc":"","ref":"GeoSQL.PostGIS.Topo.html#add_node/4"},{"type":"macro","title":"GeoSQL.PostGIS.Topo.get_face_containing_point/3","doc":"","ref":"GeoSQL.PostGIS.Topo.html#get_face_containing_point/3"},{"type":"macro","title":"GeoSQL.PostGIS.Topo.get_node_edges/3","doc":"","ref":"GeoSQL.PostGIS.Topo.html#get_node_edges/3"},{"type":"macro","title":"GeoSQL.PostGIS.Topo.get_ring_edges/3","doc":"","ref":"GeoSQL.PostGIS.Topo.html#get_ring_edges/3"},{"type":"macro","title":"GeoSQL.PostGIS.Topo.get_ring_edges/4","doc":"","ref":"GeoSQL.PostGIS.Topo.html#get_ring_edges/4"},{"type":"macro","title":"GeoSQL.PostGIS.Topo.get_topology_id/2","doc":"","ref":"GeoSQL.PostGIS.Topo.html#get_topology_id/2"},{"type":"macro","title":"GeoSQL.PostGIS.Topo.get_topology_name/2","doc":"","ref":"GeoSQL.PostGIS.Topo.html#get_topology_name/2"},{"type":"macro","title":"GeoSQL.PostGIS.Topo.get_topology_srid/2","doc":"","ref":"GeoSQL.PostGIS.Topo.html#get_topology_srid/2"},{"type":"macro","title":"GeoSQL.PostGIS.Topo.load_geometry/4","doc":"","ref":"GeoSQL.PostGIS.Topo.html#load_geometry/4"},{"type":"module","title":"GeoSQL.PostGIS.VectorTiles","doc":"In addition to support for functions related to Mapbox vector tiles, this module also\nprovides support for generating complete tiles via `generate/6`.\n\nThe `generate/6` function takes a list of `GeoSQL.PostGIS.VectorTiles.Layer` structs along\nwith the tile coordinates and an `Ecto.Repo`:\n\n  ```elixir\n    use GeoSQL.PostGIS\n\n    def tile(zoom, x, y) do\n      layers = [\n        %PostGIS.VectorTiles.Layer{\n          name: \"pois\",\n          source: \"nodes\",\n          columns: %{geometry: :geom, id: :node_id, tags: :tags}\n        },\n        %PostGIS.VectorTiles.Layer{\n          name: \"buildings\",\n          source: \"buildings\",\n          columns: %{geometry: :footprint, id: :id, tags: :tags}\n        }\n      ]\n\n\n      PostGIS.VectorTiles.generate(MyApp.Repo, zoom, x, y, layers)\n    end\n  ```\n\nThe resulting data can be loaded directly into map renderers such as `MapLibre` or `OpenLayers`\nwith the `MVT` vector tile layer format.\n\nDatabase prefixes (\"schemas\" in PostgreSQL) are also supported both on the whole tile query\nas well as per-layer.\n\nFor non-trivial tables, ensure that a `GIST` index exists on the geometry columns used.","ref":"GeoSQL.PostGIS.VectorTiles.html"},{"type":"macro","title":"GeoSQL.PostGIS.VectorTiles.as_mvt/1","doc":"","ref":"GeoSQL.PostGIS.VectorTiles.html#as_mvt/1"},{"type":"macro","title":"GeoSQL.PostGIS.VectorTiles.as_mvt/2","doc":"","ref":"GeoSQL.PostGIS.VectorTiles.html#as_mvt/2"},{"type":"macro","title":"GeoSQL.PostGIS.VectorTiles.as_mvt/3","doc":"","ref":"GeoSQL.PostGIS.VectorTiles.html#as_mvt/3"},{"type":"macro","title":"GeoSQL.PostGIS.VectorTiles.as_mvt/4","doc":"","ref":"GeoSQL.PostGIS.VectorTiles.html#as_mvt/4"},{"type":"macro","title":"GeoSQL.PostGIS.VectorTiles.as_mvt/5","doc":"","ref":"GeoSQL.PostGIS.VectorTiles.html#as_mvt/5"},{"type":"macro","title":"GeoSQL.PostGIS.VectorTiles.as_mvt_geom/2","doc":"","ref":"GeoSQL.PostGIS.VectorTiles.html#as_mvt_geom/2"},{"type":"macro","title":"GeoSQL.PostGIS.VectorTiles.as_mvt_geom/3","doc":"","ref":"GeoSQL.PostGIS.VectorTiles.html#as_mvt_geom/3"},{"type":"function","title":"GeoSQL.PostGIS.VectorTiles.generate/6","doc":"","ref":"GeoSQL.PostGIS.VectorTiles.html#generate/6"},{"type":"macro","title":"GeoSQL.PostGIS.VectorTiles.tile_envelope/3","doc":"","ref":"GeoSQL.PostGIS.VectorTiles.html#tile_envelope/3"},{"type":"macro","title":"GeoSQL.PostGIS.VectorTiles.tile_envelope/5","doc":"","ref":"GeoSQL.PostGIS.VectorTiles.html#tile_envelope/5"},{"type":"module","title":"GeoSQL.PostGIS.VectorTiles.Layer","doc":"","ref":"GeoSQL.PostGIS.VectorTiles.Layer.html"},{"type":"type","title":"GeoSQL.PostGIS.VectorTiles.Layer.column_name/0","doc":"","ref":"GeoSQL.PostGIS.VectorTiles.Layer.html#t:column_name/0"},{"type":"type","title":"GeoSQL.PostGIS.VectorTiles.Layer.columns_definition/0","doc":"","ref":"GeoSQL.PostGIS.VectorTiles.Layer.html#t:columns_definition/0"},{"type":"type","title":"GeoSQL.PostGIS.VectorTiles.Layer.t/0","doc":"","ref":"GeoSQL.PostGIS.VectorTiles.Layer.html#t:t/0"},{"type":"module","title":"GeoSQL.SpatiaLite","doc":"Non-standard GIS functions found in Spatialite.","ref":"GeoSQL.SpatiaLite.html"},{"type":"macro","title":"GeoSQL.SpatiaLite.as_gpb/1","doc":"","ref":"GeoSQL.SpatiaLite.html#as_gpb/1"},{"type":"macro","title":"GeoSQL.SpatiaLite.geom_from_gpb/1","doc":"","ref":"GeoSQL.SpatiaLite.html#geom_from_gpb/1"},{"type":"macro","title":"GeoSQL.SpatiaLite.is_valid_gpb/1","doc":"","ref":"GeoSQL.SpatiaLite.html#is_valid_gpb/1"},{"type":"module","title":"GeoSQL.SpatiaLite.TypeExtension","doc":"A type extension for `ecto_sqlite3` that implements storage and retrieval of\n`geo` structs for Spatialite databases.\n\nTo activate the extension, add this to your `config.exs`:\n\n```elixir\n  config :ecto_sqlite3,\n    extensions: [GeoSQL.Spatialite.TypeExtension]\n```\n\nAlso be sure to add `load_extensions: [\"mod_spatialite\"]` to the config of\n`Ecto.Repo`s which use the `Ecto.Adapters.SQLite3` adapter to ensure Spatialite\nfeatures are available.","ref":"GeoSQL.SpatiaLite.TypeExtension.html"},{"type":"function","title":"GeoSQL.SpatiaLite.TypeExtension.decode_geometry/1","doc":"","ref":"GeoSQL.SpatiaLite.TypeExtension.html#decode_geometry/1"},{"type":"function","title":"GeoSQL.SpatiaLite.TypeExtension.decode_geopackage_geometry/1","doc":"","ref":"GeoSQL.SpatiaLite.TypeExtension.html#decode_geopackage_geometry/1"},{"type":"function","title":"GeoSQL.SpatiaLite.TypeExtension.encode_geometry/1","doc":"","ref":"GeoSQL.SpatiaLite.TypeExtension.html#encode_geometry/1"},{"type":"function","title":"GeoSQL.SpatiaLite.TypeExtension.encode_geopackage_geometry/1","doc":"","ref":"GeoSQL.SpatiaLite.TypeExtension.html#encode_geopackage_geometry/1"},{"type":"module","title":"GeoSQL.QueryUtils","doc":"Utilities to make queries easier to produce","ref":"GeoSQL.QueryUtils.html"},{"type":"function","title":"GeoSQL.QueryUtils.as_named_params/2","doc":"Turn a set of options into named parameters using the `=>` parameter naming operator,\nchecking that the keys are allowed.\n\nThis can only be used with functions that actually have named paramters, which is mostly\nuser-provided functions.","ref":"GeoSQL.QueryUtils.html#as_named_params/2"},{"type":"function","title":"GeoSQL.QueryUtils.as_positional_params/2","doc":"Turn a set of options into positional parameters, checking that the keys are allowed.\n\nThis makes it easier to implement support for SQL functions which support a variable number\nof options (often flags to enable/disable functionality). The user can provide only the\nparameters they wish to provide, and this will turn them into a template string with the\nnecessary number of `?`s and the values included.\n\nThe `allowed_keys` must be provided in the order they appear in the SQL function.","ref":"GeoSQL.QueryUtils.html#as_positional_params/2"},{"type":"function","title":"Example - GeoSQL.QueryUtils.as_positional_params/2","doc":"For an SQL function `my_func(int foo, string bar)` with variant\n`my_func(int foo, stering bar, boolean baz)`:\n\n    foo = 100\n    valid_params = [:bar, :baz]\n    {param_string, params} = GeoSQL.QueryUtils.as_positional_params([foo: 100, bar: \"names\"], valid_params)\n    {\", ?\", [\"names\"]}\n    from(r in Schemna, where: fragment()\"my_func(?#{param_string})\", [foo | params])","ref":"GeoSQL.QueryUtils.html#as_positional_params/2-example"},{"type":"macro","title":"GeoSQL.QueryUtils.cast_to_geometry/2","doc":"Used to cast data to geometries in a backend-neutral fashion. This can be necessary\nwhen, for example, passing geogrpahy types to geometry functions in PostGIS.","ref":"GeoSQL.QueryUtils.html#cast_to_geometry/2"},{"type":"function","title":"GeoSQL.QueryUtils.decode_geometry/2","doc":"When returning a single geometry in an Ecto query `select` clause, geometries may\nbe returned as raw binaries in some backends.\n\nTo get `Geo` structs from such queries, pass the results of the query to `decode_geometry/2`\nto guarantee corectly decoded geometries regardless of the backend being used.\n\n```elixir\n      from(location in Location, select: MM.boundary(location.geom))\n      |> Repo.all()\n      |> QueryUtils.decode_geometry(Repo)\n```","ref":"GeoSQL.QueryUtils.html#decode_geometry/2"},{"type":"function","title":"GeoSQL.QueryUtils.decode_geometry/3","doc":"When returning geometries as part of a map or list Ecto query `select` clause (as opposed to\nreturning an `Ecto.Schema` struct, where geometries are automatically decoded), geometries may\nbe returned as raw binaries in some backends.\n\nTo get `Geo` structs from these queries, pass the results of the query to `decode_geometry/3`.\n\nThe third `fields_to_decode` parameter is a list of the fields to decode. When the query\nresults are lists or tuples, the `fields_to_decode` must be a list of integers that are the indexes\nof the elements to decode. In the case of maps, `fields_to_decode` should be the keys (strings or\natoms) in the map that contain geometry data.\n\nThis function is intentionally liberal with what it accepts. Instead of creating errors, it will\ninstead return the original data in the fields that failed to decode. As real-world query returns\ncan be noisy and differ between backends, this is a more usable approach in practice than returning\neither error tuples or letting exceptions bubble up.\n\n```elixir\n      from(location in Location, select: [location.name, MM.boundary(location.geom)])\n      |> Repo.all()\n      |> QueryUtils.decode_geometry(Repo, [1])\n```\n\nWhen returning a map, `fields_to_decode` must be a list of field names:\n\n\n```elixir\n      from(location in Location, select: %{name: location.name, boundary: MM.boundary(location.geom)]})\n      |> Repo.all()\n      |> QueryUtils.decode_geometry(Repo, [\"boundary])\n```","ref":"GeoSQL.QueryUtils.html#decode_geometry/3"},{"type":"macro","title":"GeoSQL.QueryUtils.wrap_wkb/2","doc":"Used to wrap WKB literals in a portable fashion. When used in an ecto query,\nthe return value will need to be pinned in the query.\n\nExample:\n\n    from(g in GeoType,\n        select: g.linestring == MM.linestring_from_wkb(^QueryUtils.wrap_wkb(wkb, MyApp.Repo))","ref":"GeoSQL.QueryUtils.html#wrap_wkb/2"},{"type":"module","title":"GeoSQL.QueryUtils.WKB","doc":"A wrapper around WKB binaries to allow them to be differentiated from other binaries.\n\nMostly useful for type adapters.\n\nSee `GeoSQL.QueryUtils.wrap_wkb/2`","ref":"GeoSQL.QueryUtils.WKB.html"},{"type":"type","title":"GeoSQL.QueryUtils.WKB.t/0","doc":"","ref":"GeoSQL.QueryUtils.WKB.html#t:t/0"},{"type":"module","title":"GeoSQL.RepoUtils","doc":"Helpers for working with Ecto Repo modules.","ref":"GeoSQL.RepoUtils.html"},{"type":"macro","title":"GeoSQL.RepoUtils.adapter/1","doc":"Macro version of `adapter_for/1` for use where a function is not\nusable or less ergonomic.","ref":"GeoSQL.RepoUtils.html#adapter/1"},{"type":"function","title":"GeoSQL.RepoUtils.adapter_for/1","doc":"Returns an adapter for a given repo, with a sensible default on `nil`.\n\nAs the details of a given SQL call may differ from implementation\nto implentation, many of the functions in GeoSQL rely on examining\nthe repo module to determine which variant to utilize.\n\nThis function provides a convenience to discover the backend\nadapter given a repo. If no adapter is provided, it defaults to\nthe configured preference, ultimately falling back to PostgreSQL.\n\nTo configure the default, adapt this line in `config.exs`:\n  config :geo_sql, default_adapter: Ecto.Adapters.","ref":"GeoSQL.RepoUtils.html#adapter_for/1"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nThis project adheres to [Semantic Versioning](http://semver.org/spec/v2.0.0.html).\n\n## [1.3.0] 21-07-2025\n\n* Improvements\n  * Introduce the `GeoSQL.Geometry.Geopackage` Ecto type, allowing Ecto schemas to\n    have geometry fields backed by Geopackage data. Currently only supported with\n    SQLite3 databases, and relies on the Spatialite module being available.\n\n## [1.2.0] 20-07-2025\n\n* Improvements\n  * Support for SpatiaLite's Geopackage interoperability functions:\n    * `GeoSQL.SpatiaLite.as_gpb`: convert a geometry to a Geopackage binary\n    * `GeoSQL.SpatiaLite.geom_from_gpb`: convert a Geopackage binary to a geometry\n    * `GeoSQL.SpatiaLite.is_valid_gpb`: checks validity of a Geopackage blob\n  * More tests\n* Fixes\n  * Standardized on the proper spelling of SpatiaLite\n\n## [1.1.0] 09-07-2025\n\n* Improvements\n  * `QueryUtils.decode_geometry` now supports tuples in addition to maps and lists.\n  * Expanded unit test coverage.\n* Fixes\n  * `MM.perimeter` had misleading parameter names.\n\n## [1.0.0] 09-07-2025\n\n* Breaking changes\n  * All SQL/MM functions were collapsed into a single MM module.\n    Migration: Replace all instances of `MM2` and `MM3` with `MM`.\n* Improvements\n  * Expanded unit test suite considerably, improved vector tile generation tests\n* Fixes\n  * `MM.locate_along` was missing a parameter placeholder\n\n## [0.1.0] 02-07-2025\n\nFirst release.\n\n* Spatial database support\n  * PostGIS\n  * Spatialite\n* GIS SQL functions (290+)\n  * SQL/MM functions (part 2 and 3)\n  * Common, but non-standard functions\n  * Non-stanadard PostGIS functions\n  * Mapbox vector tile generation (PostGIS only)\n* Eco Schema types\n  * GeoSQL.Geometry - supports any geometry\n  * GeoSQL.Geography - suports any geography\n  * GeoSQL.Geometry.  - specific Geometry types (points, lines, polygons, etc.)\n  * GeoSQL.Int4","ref":"changelog.html"},{"type":"extras","title":"GeoSQL","doc":"# GeoSQL\n\nThis library provides access to geometric and geographic SQL functions as\nimplemented in SQL extensions such as PostGIS and SpatiaLite.\n\nThis includes the entire suite of SQL/MM spatial functions,\nnon-standard functions that are found in commonly used GIS-enabled databases,\nimplementation-specific functions found in specific backends, as well as high-\nlevel functions for features such as generating Mapbox vector tiles.\n\nThe goals of this library are:\n\n * Ease: fast to get started, hide complexity where possible\n * Portability: currently supports PostGIS and SpatiaLite.\n * Completeness: extensive support for GIS SQL functions, not just the most common ones.\n * Clarity: Functions organized by their availability and standards compliance\n * Utility: Provide out-of-the-box support for complete worfklows. Mapbox vector tile\n   generation is a good example: one call to `GeoSQL.PostGIS.VectorTiles.generate/6`\n   is enough to retrieve complete vector tiles based on any table in the database that has a geometry field.\n\nNot-goals include:\n\n * Having the fewest possible dependencies. Ecto adapters are pulled in as necessary,\n   along with other dependencies such as `Jason` in order to ease use.","ref":"readme.html"},{"type":"extras","title":"Usage - GeoSQL","doc":"Add `GeoSQL` to your project by adding the following to the `deps` section in `mix.exs` (or equivalent):\n\n  ```\n  {:geo_sql, \"~> 0.1\"}\n  ```\n\nRun the usual `mix deps.get`!\n\nFull documentation can be generated locally with `mix docs`.","ref":"readme.html#usage"},{"type":"extras","title":"Ecto Schemas - GeoSQL","doc":"Ecto Schemas can have fields with the following values:\n\n  * GeoSQL.Geometry: this supports *all* geometry and geography types. It does no typechecking\n    beyond confirming it is a Geo-compatible type, making it a perfect \"catch-all\" generic\n    type for use in schemas.\n  * GeoSQL.Geometry.Point\n  * GeoSQL.Geometry.PointZ\n  * GeoSQL.Geometry.PointM\n  * GeoSQL.Geometry.PointZM\n  * GeoSQL.Geometry.LineString\n  * GeoSQL.Geometry.LineStringZ\n  * GeoSQL.Geometry.LineStringZM\n  * GeoSQL.Geometry.Polygon\n  * GeoSQL.Geometry.PolygonZ\n  * GeoSQL.Geometry.MultiPoint\n  * GeoSQL.Geometry.MultiPointZ\n  * GeoSQL.Geometry.MultiLineString\n  * GeoSQL.Geometry.MultiLineStringZ\n  * GeoSQL.Geometry.MultiLineStringZM\n  * GeoSQL.Geometry.MultiPolygon\n  * GeoSQL.Geometry.MultiPolygonZ\n  * GeoSQL.Geometry.GeometryCollection\n\nExample:\n\n  ```elixir\n  defmodule MyApp.GeoTable do\n    use Ecto.Schema\n\n    schema \"specified_columns\" do\n      field(:name, :string)\n      field(:geometry, GeoSQL.Geometry) # will match any Geo type\n      field(:point, GeoSQL.Geometry.Point) # will reject any non-Point data\n      field(:linestring, GeoSQL.Geometry.LineStringZ) # will reject any non-LineStringZ data\n    end\n  end\n  ```\n\n#### Geopackage\n\nThe Geopackage standard defines its own binary format for serializing geometries in SQLite3\ndatabases. GeoSQL provides access to these types via the `GeoSQL.Geometry.Geopackage` type\nallowing Ecto schemas to contain geometry fields that are stored as Geopackage data.\n\nExample:\n\n  ```elixir\n  defmodule GeoSQL.Test.Schema.Geopackage do\n    use Ecto.Schema\n\n    @primary_key false\n    schema \"some_table_in_geopackage\" do\n      field(:id, :integer, source: :OBJECTID)\n      field(:name, :string)\n      field(:shape, GeoSQL.Geometry.Geopackage, source: :Shape)\n    end\n  end\n  ```\n\nThis schema can now be used in queries like any other:\n\n  ```elixir\n  from(g in Geopackage) |> GeopackageRepo.all()\n  ```\n\nDue to being tied to SQLite3, only SQLite3 databases are supported and the Spatialite\nmodule must be available as it is (currently) used to do the serialization in memory.\nThe Spatialite module does not need to be initialized on a Geopackage database, but\nit does need to be available on the system for GeoSQL to use.\n\nNOTE: If the `Geometry` library gains support for the Geopackage binary format, similar to\nits suport for the WKB format, then the Spatialite requirement may be dropped.","ref":"readme.html#ecto-schemas"},{"type":"extras","title":"Readying the Repo with GeoSQL.init/1 - GeoSQL","doc":"Once added to your project, an `Ecto.Repo` can be readied for use by calling\n`GeoSQL.init/2`. This can be done once the repo has been started by implementing\nthe `init/2` callback in your repo module like this:\n\n  ```elixir\n  defmodule MyApp.Repo do\n    use Ecto.Repo, otp_app: :my_app, adapter: Ecto.Adapters.Postgres\n\n    @impl true\n    def init(:supervisor, config) do\n      GeoSQL.init(__MODULE__, json: Jason)\n      {:ok, config}\n    end\n\n    def init(:runtime, config), do: {:ok, config}\n  end\n  ```\n\nFor PostGIS, types are automatically added by `GeoSQL.init/2`. For this reason, call\n`GeoSQL.init/2` *after* any other custom types are registered. The type extensions included\nin `GeoSQL` are available via the `GeoSQL.PostGIS.Extension.extensions/0` function.\n\nIf migrations are failing, place a `GeoSQL.init/1` call in the top-level of the file the Repo is defined in:\n\n  ```\n  defmodule MyApp.Repo do\n    use Ecto.Repo, otp_app: :geo_sql, adapter: Ecto.Adapters.Postgres\n  end\n\n  # For the migrations.\n  GeoSQL.init(MyApp.Repo)\n  ```\n\nThis will ensure that any special types are defined and registered, though it\nwill still need to be called after the repo has been started.\n\nDynamic Ecto repositories are also supported, and `GeoSQL.init/1` can be\ncalled after the call to `Repo.put_dynamic_repo/1` has completed.","ref":"readme.html#readying-the-repo-with-geosql-init-1"},{"type":"extras","title":"Macro usage - GeoSQL","doc":"Once initialized, the wide array of macros can be used with `Ecto` queries:\n\n  ```elixir\n  from(location in Location, select: Common.extent(location.geom, MyApp.Repo))\n  ```\n\nSome macros, such as `GeoSQL.Common.extent`, take an optional `Ecto.Repo` parameter.\nThis allows those macros to generate the correct SQL statements for the backend being used.\nIf no repo is passed to those functions, they assume PostGIS compatibility by default, though\nthis can be configured by adding this to `config.ex`:\n\n  ```elixir\n  config :geo_sql, default_adapter: Ecto.Adapters. \n  ```\n\nNote that the value passed must be the literal repo module name. Passing in a variable\nto which the repo was assigned will usually fail unless wrapped in a macro context, as\nEcto does all of its magic at compile-time, making the value of runtime variables unnavailable\nfor constructing queries (which is different from populating them with values). Usually this is\nnot an issue.\n\n#### Composition\n\n`GeoSQL` macros can also be freely composted and used together, such as this query which\nuses a number of standard and PostGIS-specific features together:\n\n  ```elixir\n  from(g in layer.source,\n    prefix: ^layer.prefix,\n    where:\n      bbox_intersects?(\n        field(g, ^columns.geometry),\n        MM.transform(tile_envelope(^z, ^x, ^y), type(^layer.srid, Int4))\n      ),\n    select: %{\n      name: ^layer.name,\n      geom:\n        as_mvt_geom(\n          field(g, ^columns.geometry),\n          MM.transform(\n            tile_envelope(^z, ^x, ^y),\n            type(^layer.srid, Int4)\n          )\n        ),\n      id: field(g, ^columns.id),\n      tags: field(g, ^columns.tags)\n    }\n  )\n  ```\n\n#### Queries needing geometry type casting\n\nSometimes queries will require casting to the database's native geometry type. Such casting is backend-specific, and so the `GeoSQL.QueryUtils.cast_to_geometry/2`\nfunction which takes an `Ecto.Repo` is provided for portability.\n\nThe need to use it  occurs when, for example, a query passes a geography type to a geometry\nfunction in PostGIS, or the adapter (e.g. `postgrex`) can not automatically determine the type.\nA common symptom of the latter case are errors noting that a binary was expected, and\nthe geometry struct provided was not serialized.\n\nFor example this query, where `lineA` and `lineB` are `Geometry.LineString` structs:\n\n  ```elixir\n  from(location in Locations, select: MM.intersection(^lineA, ^lineB))\n  ```\n\nmay produce this error:\n\n  ```\n  Postgrex expected a binary, got %Geometry.LineString{path: [[30, -90], [30, -91]], srid: 4326}. Please make sure the value you are passing matches the definition in your table or in your query or convert the value accordingly.\n  ```\n\nCasting one of the two type is usually enough to resolve this:\n\n\n  ```elixir\n  from(location in Locations, select: MM.intersection(QueryUtils.cast_to_geometry(^lineA, MyApp.Repo), ^lineB))\n  ```\n\nNote that using Ecto Schemas or referencing columns from a table avoids these issues, as the\ndatabase adapters can determine what the correct types are from that information.","ref":"readme.html#macro-usage"},{"type":"extras","title":"Queries Taking WKB data - GeoSQL","doc":"Some functions take WKB-encoded data. If passing WKB blobs from the client-side to the backend,\nwrap them using the `QueryUtils.wrap_wkb/2` macro, passing in the Ecto repo as the second parameter.\n\nWhen used in a query, the return of this macro will need to be pinned (`^`) as it returns a value.\n\nExample:\n\n\n  ```elixir\n  from(g in GeoType,\n    select: g.linestring == MM.linestring_from_wkb(^QueryUtils.wrap_wkb(wkb, MyApp.Repo), ^line.srid)\n  )\n  ```","ref":"readme.html#queries-taking-wkb-data"},{"type":"extras","title":"Queries returning binary blobs instead of geometries - GeoSQL","doc":"With certain backends (e.g. SQLite3), it is possible to craft queries that will return\nbinary blobs instead of decoded `Geo` structs.\n\nIn such cases, use `GeoSQL.QueryUtils.decode_geometry/2`:\n\n  ```elixir\n  defmodule MyApp.Plots do\n    use GeoSQL.MM\n    use GeoSQL.QueryUtils\n\n    def boundaries() do\n      from(location in Location, select: MM.boundary(location.geom))\n      |> Repo.all()\n      |> QueryUtils.decode_geometry(Repo)\n    end\n  end\n  ```\n\nFor backends that do not suffer from this (e.g. PostGIS), the call to `GeoSQL.decode_geometry`\nis efficient, doing little more than a comparison of a single `atom` to determine no further\naction needs to be taken.","ref":"readme.html#queries-returning-binary-blobs-instead-of-geometries"},{"type":"extras","title":"Module organization - GeoSQL","doc":"Features are organized into modules by their availability and topic.\n\nThe v2 and v3 sets of standard SQL/MM functions for geospatial applications are found\nin the `GeoSQL.MM`module. Non-standardized but commonly implemented functions are found in the `GeoSQL.Common` namespace, while implementation-specific fuctions are found in namespaces indicating the target database (e.g. `GeoSQL.PostGIS`).\n\nTopological and 3D functions are found in `Topo` and `ThreeD` modules within this\nhierarchy, as they less-used and/or have very similar names to more commonly used\nSQL functions.\n\nThis helps make it clear what features your code relies on, allowing\none to audit feature usage for compability and avoid incompatible use\nin the first place.\n\nFor example, if targeting both `SpatiaLite` and `PostGIS`, the code should only use the standard SQL/MM features plus those in the `GeoSQL.Common` modules.\n\nTo make this even easier, each of the top-level modules supports the `use` syntax which\npulls in their suite of features and introduces helpful aliases with one line in your code:\n\n  ```elixir\n  use GeoSQL.MM\n\n  def query() do\n    from(features in MyApp.Feature\n      select: %{\n        area_2d: MM.area(features.geometry)`\n        area_3d: MM.ThreeD.area(features.geometry)\n      }\n    )\n  end\n\n  ```","ref":"readme.html#module-organization"},{"type":"extras","title":"Mapbox Vector Tiles - GeoSQL","doc":"`GeoSQL` can generate vector tiles using the Mapbox encoding directly from PostGIS databases.\nIt works with any table that has an id column, a column with geometry information, and a set of\ntagged information such as names. The tag information is usually fetch as (or from) a `jsonb` data.\n\nThe `PostGIS.VectorTiles.generate/5` function takes a layer definition in the form of a list of\n`PostGIS.VectorTiles.Layer` structs along with the tile coordinates and an `Ecto.Repo`:\n\n  ```elixir\n  def tile(zoom, x, y) do\n    layers = [\n      %PostGIS.VectorTiles.Layer{\n        name: \"pois\",\n        source: \"nodes\",\n        columns: %{geometry: :geom, id: :node_id, tags: :tags}\n      },\n      %PostGIS.VectorTiles.Layer{\n        name: \"buildings\",\n        source: \"buildings\",\n        columns: %{geometry: :footprint, id: :id, tags: :tags}\n      }\n    ]\n\n\n    PostGIS.VectorTiles.generate(MyApp.Repo, zoom, x, y, layers)\n  end\n  ```\n\nThe resulting data can be loaded directly into map renderers such as `MapLibre` or `OpenLayers`\nwith the `MVT` vector tile layer format.\n\nDatabase prefixes (\"schemas\" in PostgreSQL) are also supported both on the whole tile query\nas well as per-layer.","ref":"readme.html#mapbox-vector-tiles"},{"type":"extras","title":"Building - GeoSQL","doc":"To build and interact with the library locally:\n\n    git clone https://github.com/aseigo/geo_sql.git\n    cd geo_sql\n    mix deps.get\n    mix compile\n    iex -S mix","ref":"readme.html#building"},{"type":"extras","title":"Unit Tests - GeoSQL","doc":"Unit tests currently assume a working PostGIS installation is available locally.\n\nThe URL for the test database is defined in `config/test.exs`.\n\n**Note** that this database will be created and **dropped** on every run of the tests.\nDo NOT point it to an existing database!\n\nThe migrations in `priv/repo/migrations` are run on each test run.\n\nRunning `mix test` will run tests along with setting up and tearing down the database.\nThis allows the tests to access the database in a known state each run.\n\nTests may be run continuously with `mix test.watch`.","ref":"readme.html#unit-tests"},{"type":"extras","title":"Running tests for a subset of backends - GeoSQL","doc":"To limit which backends the tests are run against, set the `GEOSQL_TEST_BACKENDS` environment variable before running tests to a comma-separated list of backends.\n\nExample:\n\n  ```shell\n  # Run only the PostGIS tests.\n  GEOSQL_TEST_BACKENDS=pgsql mix test test/ecto_test.exs\n  ```\n\nCurrent the following backends are recognized:\n\n  * `pgsql`\n  * `sqlite3`","ref":"readme.html#running-tests-for-a-subset-of-backends"},{"type":"extras","title":"Contributing - GeoSQL","doc":"If you would like to contribute support for more functions (PostGIS and\nSpatiaLite both provide a frighteningly impressive amount of them!) or\nsupport for other databases, do not hesitate to make a PR and the author\nwill review and merge in a timely fashion.","ref":"readme.html#contributing"},{"type":"extras","title":"Acknowledgements - GeoSQL","doc":"This library began as a fork of the excellent `geo_postgis`, which the author\nhas used for many years, before growing into something rather larger. A big thank-you to felt.com for maintaining that\nlibrary over the course of many years.","ref":"readme.html#acknowledgements"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}